﻿using AutoMapper;
using Infragistics.Web.Mvc;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;

namespace infragistic_generic_grid
{
    public class HomeController : Controller
    {
        ApplicationContext context = new ApplicationContext();

        IMapper mapper;

        public HomeController()
        {
            var mapperConfig = new MapperConfiguration(cfg =>
            {
                cfg.AddProfile(new MappingProfile());
            });
            mapper = mapperConfig.CreateMapper();
        }

        public ActionResult Index()
        {
            return View();
        }

        public ActionResult GenericGridPage()
        {
            return View();
        }

        [GridDataSourceAction]
        public ActionResult SaveGenericData()
        {
            GridModel gridModel = new GridModel();

            // Get the grid transaction
            List<Transaction<TestCountry>> transactions = gridModel.LoadTransactions<TestCountry>(HttpContext.Request.Form["ig_transactions"]);

            // For each modification
            foreach (Transaction<TestCountry> t in transactions)
            {
                if (t.type == "newrow")
                {
                    context.TestCountries.Add(t.row);
                }
                else if (t.type == "deleterow")
                {
                    int id = Int32.Parse(t.rowId);
                    var country = context.TestCountries.Single(c => c.CountryId == id);
                    context.TestCountries.Remove(country);
                }
                else if (t.type == "row")
                {
                    int id = Int32.Parse(t.rowId);
                    var country = context.TestCountries.Single(c => c.CountryId == id);

                    if (t.row.Code != null)
                    {
                        country.Code = t.row.Code;
                    }
                    if (t.row.Name != null)
                    {
                        country.Name = t.row.Name;
                    }
                }
            }

            context.SaveChanges();

            JsonResult result = new JsonResult();
            Dictionary<string, bool> response = new Dictionary<string, bool>();
            response.Add("Success", true);
            result.Data = response;
            return result;
        }

        [GridDataSourceAction]
        public ActionResult GetGenericData()
        {
            var countryEntites = context.TestCountries.OrderBy(o => o.CountryId).Take(20).AsQueryable();

            return View(countryEntites);

        }

    }

    public class MappingProfile : Profile
    {
        protected override void Configure()
        {
            CreateMap<TestCountry, TestCountryModel>().ReverseMap();
        }
    }

    public class IgGridResult<T> : JsonResult
    {
        public IgGridResult(IEnumerable<T> records)
        {
            this.JsonRequestBehavior = JsonRequestBehavior.AllowGet; // Infragistics grid perform GET request with Json responses
            this.Data = new
            {
                Records = records.ToArray() // see Infragistics.Web.Mvc.WrappedGridResponse
            };
            this.MaxJsonLength = 20000000; // 20 MB
        }
    }

}